@@ -23,18 +23,17 @@ struct wlr_backend
     ...;
 };
 
-typedef int EGLint;
-typedef unsigned int EGLenum;
-
-typedef struct wlr_renderer *(*wlr_renderer_create_func_t)(struct wlr_egl *egl, EGLenum platform,
-    void *remote_display, EGLint *config_attribs, EGLint visual_id);
-
-struct wlr_backend *wlr_backend_autocreate(struct wl_display *display,
-    wlr_renderer_create_func_t create_renderer_func);
+struct wlr_backend *wlr_backend_autocreate(struct wl_display *display);
 
 bool wlr_backend_start(struct wlr_backend *backend);
 void wlr_backend_destroy(struct wlr_backend *backend);
 struct wlr_renderer *wlr_backend_get_renderer(struct wlr_backend *backend);
+struct wlr_session *wlr_backend_get_session(struct wlr_backend *backend);
+"""
+
+# backend/session.h
+CDEF += """
+bool wlr_session_change_vt(struct wlr_session *session, unsigned vt);
 """
 
 # render/wlr_renderer.h
@@ -48,7 +47,21 @@ bool wlr_render_texture(struct wlr_renderer *r, struct wlr_texture *texture,
 bool wlr_render_texture_with_matrix(struct wlr_renderer *r,
     struct wlr_texture *texture, const float matrix[static 9], float alpha);
 
+void wlr_render_rect(struct wlr_renderer *r, const struct wlr_box *box,
+    const float color[static 4], const float projection[static 9]);
+
+const enum wl_shm_format *wlr_renderer_get_shm_texture_formats(
+    struct wlr_renderer *r, size_t *len);
+
 void wlr_renderer_init_wl_display(struct wlr_renderer *r, struct wl_display *wl_display);
+void wlr_renderer_destroy(struct wlr_renderer *renderer);
+"""
+
+# render/wlr_texture.h
+CDEF += """
+struct wlr_texture *wlr_texture_from_pixels(struct wlr_renderer *renderer,
+    uint32_t fmt, uint32_t stride, uint32_t width, uint32_t height,
+    const void *data);
 """
 
 # types/wlr_box.h
@@ -58,6 +71,27 @@ struct wlr_box {
     int width, height;
     ...;
 };
+
+struct wlr_fbox {
+    double x, y;
+    double width, height;
+    ...;
+};
+
+void wlr_box_closest_point(const struct wlr_box *box, double x, double y,
+    double *dest_x, double *dest_y);
+
+bool wlr_box_intersection(struct wlr_box *dest, const struct wlr_box *box_a,
+    const struct wlr_box *box_b);
+
+bool wlr_box_contains_point(const struct wlr_box *box, double x, double y);
+
+bool wlr_box_empty(const struct wlr_box *box);
+
+void wlr_box_transform(struct wlr_box *dest, const struct wlr_box *box,
+    enum wl_output_transform transform, int width, int height);
+
+void wlr_box_rotated_bounds(struct wlr_box *dest, const struct wlr_box *box, float rotation);
 """
 
 # types/wlr_cursor.h
@@ -99,6 +133,8 @@ void wlr_cursor_destroy(struct wlr_cursor *cur);
 
 bool wlr_cursor_warp(struct wlr_cursor *cur, struct wlr_input_device *dev,
     double lx, double ly);
+void wlr_cursor_warp_closest(struct wlr_cursor *cur,
+    struct wlr_input_device *dev, double x, double y);
 void wlr_cursor_warp_absolute(struct wlr_cursor *cur,
     struct wlr_input_device *dev, double x, double y);
 void wlr_cursor_move(struct wlr_cursor *cur, struct wlr_input_device *dev,
@@ -126,6 +162,25 @@ void wlr_seat_set_selection(struct wlr_seat *seat,
     struct wlr_data_source *source, uint32_t serial);
 """
 
+# types/wlr_gamma_control_v1.h
+CDEF += """
+struct wlr_gamma_control_manager_v1 {
+    struct wl_global *global;
+    struct wl_list controls; // wlr_gamma_control_v1::link
+
+    struct wl_listener display_destroy;
+
+    struct {
+        struct wl_signal destroy;
+    } events;
+
+    void *data;
+    ...;
+};
+struct wlr_gamma_control_manager_v1 *wlr_gamma_control_manager_v1_create(
+    struct wl_display *display);
+"""
+
 # types/wlr_input_device.h
 CDEF += """
 enum wlr_button_state {
@@ -240,17 +295,11 @@ struct wlr_keyboard {
     ...;
 };
 
-enum wlr_key_state {
-    WLR_KEY_RELEASED,
-    WLR_KEY_PRESSED,
-    ...
-};
-
 struct wlr_event_keyboard_key {
     uint32_t time_msec;
     uint32_t keycode;
     bool update_state;
-    enum wlr_key_state state;
+    enum wl_keyboard_key_state state;
     ...;
 };
 
@@ -329,7 +378,6 @@ struct wlr_output {
     enum wl_output_transform transform;
 
     bool needs_frame;
-    struct pixman_region32 damage;
     bool frame_pending;
     float transform_matrix[9];
 
@@ -337,14 +385,14 @@ struct wlr_output {
 
     struct {
         struct wl_signal frame;
+        struct wl_signal damage;
         struct wl_signal needs_frame;
         struct wl_signal precommit;
         struct wl_signal commit;
         struct wl_signal present;
+        struct wl_signal bind;
         struct wl_signal enable;
         struct wl_signal mode;
-        struct wl_signal scale;
-        struct wl_signal transform;
         struct wl_signal description;
         struct wl_signal destroy;
     } events;
@@ -399,6 +447,28 @@ void wlr_output_layout_add_auto(struct wlr_output_layout *layout,
 
 # types/wlr_pointer.h
 CDEF += """
+struct wlr_pointer {
+    const struct wlr_pointer_impl *impl;
+
+    struct {
+        struct wl_signal motion;
+        struct wl_signal motion_absolute;
+        struct wl_signal button;
+        struct wl_signal axis;
+        struct wl_signal frame;
+        struct wl_signal swipe_begin;
+        struct wl_signal swipe_update;
+        struct wl_signal swipe_end;
+        struct wl_signal pinch_begin;
+        struct wl_signal pinch_update;
+        struct wl_signal pinch_end;
+    } events;
+
+    void *data;
+
+    ...;
+};
+
 struct wlr_event_pointer_motion {
     struct wlr_input_device *device;
     uint32_t time_msec;
@@ -445,6 +515,72 @@ struct wlr_event_pointer_axis {
     int32_t delta_discrete;
     ...;
 };
+
+struct wlr_event_pointer_swipe_begin {
+    struct wlr_input_device *device;
+    uint32_t time_msec;
+    uint32_t fingers;
+    ...;
+};
+
+struct wlr_event_pointer_swipe_update {
+    struct wlr_input_device *device;
+    uint32_t time_msec;
+    uint32_t fingers;
+    double dx, dy;
+    ...;
+};
+
+struct wlr_event_pointer_swipe_end {
+    struct wlr_input_device *device;
+    uint32_t time_msec;
+    bool cancelled;
+    ...;
+};
+
+struct wlr_event_pointer_pinch_begin {
+    struct wlr_input_device *device;
+    uint32_t time_msec;
+    uint32_t fingers;
+    ...;
+};
+
+struct wlr_event_pointer_pinch_update {
+    struct wlr_input_device *device;
+    uint32_t time_msec;
+    uint32_t fingers;
+    double dx, dy;
+    double scale;
+    double rotation;
+    ...;
+};
+
+struct wlr_event_pointer_pinch_end {
+    struct wlr_input_device *device;
+    uint32_t time_msec;
+    bool cancelled;
+    ...;
+};
+"""
+
+# types/wlr_screencopy_v1.h
+CDEF += """
+struct wlr_screencopy_manager_v1 {
+    struct wl_global *global;
+    struct wl_list frames; // wlr_screencopy_frame_v1::link
+
+    struct wl_listener display_destroy;
+
+    struct {
+        struct wl_signal destroy;
+    } events;
+
+    void *data;
+    ...;
+};
+
+struct wlr_screencopy_manager_v1 *wlr_screencopy_manager_v1_create(
+    struct wl_display *display);
 """
 
 # types/wlr_seat.h
@@ -669,7 +805,7 @@ struct wlr_surface_role {
 struct wlr_surface {
     struct wl_resource *resource;
     struct wlr_renderer *renderer;
-    struct wlr_buffer *buffer;
+    struct wlr_client_buffer *buffer;
     int sx, sy;
     struct pixman_region32 buffer_damage;
     struct pixman_region32 opaque_region;
@@ -704,6 +840,39 @@ typedef void (*wlr_surface_iterator_func_t)(struct wlr_surface *surface,
 extern "Python" void surface_iterator_callback(struct wlr_surface *surface, int sx, int sy, void *data);
 """
 
+# types/wlr_virtual_keyboard_v1.h
+CDEF += """
+struct wlr_virtual_keyboard_manager_v1 {
+    struct wl_global *global;
+    struct wl_list virtual_keyboards; // struct wlr_virtual_keyboard_v1*
+
+    struct wl_listener display_destroy;
+
+    struct {
+        struct wl_signal new_virtual_keyboard; // struct wlr_virtual_keyboard_v1*
+        struct wl_signal destroy;
+    } events;
+    ...;
+};
+
+struct wlr_virtual_keyboard_v1 {
+    struct wlr_input_device input_device;
+    struct wl_resource *resource;
+    struct wlr_seat *seat;
+    bool has_keymap;
+
+    struct wl_list link;
+
+    struct {
+        struct wl_signal destroy; // struct wlr_virtual_keyboard_v1*
+    } events;
+    ...;
+};
+
+struct wlr_virtual_keyboard_manager_v1* wlr_virtual_keyboard_manager_v1_create(
+    struct wl_display *display);
+"""
+
 # types/wlr_xcursor_manager.h
 CDEF += """
 struct wlr_xcursor_manager *wlr_xcursor_manager_create(const char *name,
@@ -716,6 +885,29 @@ void wlr_xcursor_manager_set_cursor_image(struct wlr_xcursor_manager *manager,
     const char *name, struct wlr_cursor *cursor);
 """
 
+# types/wlr_xdg_output_v1.h
+CDEF += """
+struct wlr_xdg_output_manager_v1 {
+    struct wl_global *global;
+    struct wlr_output_layout *layout;
+
+    struct wl_list outputs;
+
+    struct {
+        struct wl_signal destroy;
+    } events;
+
+    struct wl_listener display_destroy;
+    struct wl_listener layout_add;
+    struct wl_listener layout_change;
+    struct wl_listener layout_destroy;
+    ...;
+};
+
+struct wlr_xdg_output_manager_v1 *wlr_xdg_output_manager_v1_create(
+    struct wl_display *display, struct wlr_output_layout *layout);
+"""
+
 # types/wlr_xdg_shell.h
 CDEF += """
 struct wlr_xdg_shell {
@@ -889,7 +1081,7 @@ void wlr_xdg_surface_get_geometry(struct wlr_xdg_surface *surface,
 void wlr_xdg_surface_for_each_surface(struct wlr_xdg_surface *surface,
     wlr_surface_iterator_func_t iterator, void *user_data);
 
-void wlr_xdg_surface_for_each_popup(struct wlr_xdg_surface *surface,
+void wlr_xdg_surface_for_each_popup_surface(struct wlr_xdg_surface *surface,
     wlr_surface_iterator_func_t iterator, void *user_data);
 """
 
@@ -922,6 +1114,24 @@ typedef void (*wrapped_log_func_t)(enum wlr_log_importance importance, const cha
 void wrapped_log_init(enum wlr_log_importance verbosity, wrapped_log_func_t callback);
 """
 
+# backend/headless.h
+CDEF += """
+struct wlr_backend *wlr_headless_backend_create(struct wl_display *display);
+
+struct wlr_backend *wlr_headless_backend_create_with_renderer(
+    struct wl_display *display, struct wlr_renderer *renderer);
+
+struct wlr_output *wlr_headless_add_output(struct wlr_backend *backend,
+    unsigned int width, unsigned int height);
+
+struct wlr_input_device *wlr_headless_add_input_device(
+    struct wlr_backend *backend, enum wlr_input_device_type type);
+
+bool wlr_backend_is_headless(struct wlr_backend *backend);
+bool wlr_input_device_is_headless(struct wlr_input_device *device);
+bool wlr_output_is_headless(struct wlr_output *output);
+"""
+
 # version.h
 CDEF += """
 #define WLR_VERSION_MAJOR ...
@@ -931,17 +1141,22 @@ CDEF += """
 
 SOURCE = """
 #include <wlr/backend.h>
+#include <wlr/backend/headless.h>
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_data_device.h>
+#include <wlr/types/wlr_gamma_control_v1.h>
 #include <wlr/types/wlr_keyboard.h>
 #include <wlr/types/wlr_linux_dmabuf_v1.h>
 #include <wlr/types/wlr_matrix.h>
 #include <wlr/types/wlr_output.h>
 #include <wlr/types/wlr_output_layout.h>
+#include <wlr/types/wlr_screencopy_v1.h>
 #include <wlr/types/wlr_seat.h>
+#include <wlr/types/wlr_virtual_keyboard_v1.h>
 #include <wlr/types/wlr_xcursor_manager.h>
+#include <wlr/types/wlr_xdg_output_v1.h>
 #include <wlr/types/wlr_xdg_shell.h>
 #include <wlr/util/log.h>
 #include <wlr/version.h>
