@@ -4,6 +4,7 @@ from .box import Box  # noqa: F401
 from .compositor import Compositor  # noqa: F401
 from .cursor import Cursor  # noqa: F401
 from .data_device_manager import DataDeviceManager  # noqa: F401
+from .gamma_control_v1 import GammaControlManagerV1  # noqa: F401
 from .input_device import InputDevice  # noqa: F401
 from .keyboard import Keyboard  # noqa: F401
 from .matrix import Matrix  # noqa: F401
@@ -15,8 +16,11 @@ from .pointer import (  # noqa: F401
     PointerEventMotion,
     PointerEventMotionAbsolute,
 )
+from .screencopy_v1 import ScreencopyManagerV1  # noqa: F401
 from .seat import Seat  # noqa: F401
 from .surface import Surface, SurfaceState  # noqa: F401
 from .texture import Texture  # noqa: F401
+from .virtual_keyboard_v1 import VirtualKeyboardManagerV1  # noqa: F401
 from .xcursor_manager import XCursorManager  # noqa: F401
+from .xdg_output_v1 import XdgOutputManagerV1  # noqa: F401
 from .xdg_shell import XdgShell  # noqa: F401
